import logging
from unittest.mock import call, patch

import pytest

from curvenote.latex.utils import just_log_errors, log_and_raise_errors


@patch.object(logging, "info", wraps=logging.info)
def test_just_log_value_errors_factory(mock_logging):
    @just_log_errors(lambda *args: f"message {args[0]}")
    def under_test(*args):
        raise ValueError("boom")

    under_test(1)
    mock_logging.assert_called_with("message 1")


@patch.object(logging, "error", wraps=logging.error)
def test_just_log_value_errors(mock_logging):
    @log_and_raise_errors(lambda *args: f"message {args[0]}")
    def under_test(*args):
        raise ValueError("boom")

    with pytest.raises(ValueError):
        under_test(1)
    mock_logging.assert_has_calls([call("message 1"), call("Error: %s", "boom")])



from unittest.mock import MagicMock, Mock, patch

import pytest

from curvenote.client import Session
from curvenote.latex import LatexProject
from curvenote.models import BlockFormat


@patch.object(LatexProject, "create_folders")
def test_create_project(create_folders):
    LatexProject(Mock(), "some/folder", {})

    assert create_folders.called


@patch.object(LatexProject, "add_article")
@patch.object(LatexProject, "reconcile")
@patch.object(LatexProject, "create_folders")
def test_build_single_article_by_name(add_article, reconcile, create_folders):
    LatexProject.build_single_article_by_name(
        "some/folder",
        Session("fake-token"),
        "some-project-id",
        "some-article-id",
        1,
        BlockFormat.tex,
        {},
    )

    assert create_folders.called
    assert add_article.called
    assert reconcile.called



import pytest
from curvenote_template.options import (
    AsideSchemaOption,
    SchemaOptionDefs,
    StringSchemaOption,
)


def test_empty_option():
    opts = StringSchemaOption()
    with pytest.raises(ValueError) as err:
        opts.get()
    assert "No default option set" in str(err)


def test_unknown_option():
    opts = StringSchemaOption()
    with pytest.raises(ValueError) as err:
        opts.get("something")
    assert "Unknown option: something" in str(err)


def test_add_option():
    opts = StringSchemaOption()
    opts.add("a", SchemaOptionDefs(["a"], ["b"], ["c"]))
    assert opts.get("a").passopts == ["a"]
    assert opts.get("a").packages == ["b"]
    assert opts.get("a").setup == ["c"]


def test_default_option():
    opts = StringSchemaOption()
    opts.add("a", SchemaOptionDefs(["a"], ["b"], ["c"]))
    opts.default = "a"
    assert opts.get().passopts == ["a"]
    assert opts.get().packages == ["b"]
    assert opts.get().setup == ["c"]


def test_names():
    opts = StringSchemaOption()
    opts.add("a", SchemaOptionDefs())
    opts.add("b", SchemaOptionDefs())
    assert opts.names() == ["a", "b"]


def test_aside_options():
    aside = AsideSchemaOption()

    assert aside.default == "marginpar"
    assert "marginpar" in aside.names()
    assert "framed" in aside.names()
    assert "mdframed" in aside.names()
    assert "callout" in aside.names()
    assert "aside.def" in aside.names()



import re

import pytest

from curvenote.client import API_URL
from curvenote.latex import utils
from curvenote.latex.utils.index import LocalMarker


def test_block_hash_to_url():
    block_hash = "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E"
    url = utils.block_hash_to_api_url(API_URL, block_hash)
    assert url == f"{API_URL}/blocks/VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E"


def test_unpinned_block_hash_to_url():
    block_hash = "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1"
    url = utils.block_hash_to_api_url(API_URL, block_hash)
    assert (
        url == f"{API_URL}/blocks/VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/versions/1"
    )


def test_unpinned_oxa_path_to_url():
    oxa_path = "oxa:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E"
    url = utils.oxa_path_to_api_url(API_URL, oxa_path)
    assert url == f"{API_URL}/blocks/VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E"


def test_oxa_path_to_url():
    oxa_path = "oxa:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E.1"
    url = utils.oxa_path_to_api_url(API_URL, oxa_path)
    assert (
        url == f"{API_URL}/blocks/VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/versions/1"
    )


def test_find_no_images_in_tex():
    content = """
        lorem ipsum anythingim
        \\begin{center}
        \\somthing{abc}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """
    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )
    assert summary.content == content
    assert summary.remote_paths == []
    assert summary.local_paths == []


def test_find_one_image_in_bigger_tex_block__block_path():
    content = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    updated = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )

    assert summary.content == updated
    assert summary.remote_paths == ["block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1"]
    assert summary.local_paths == ["images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1"]


def test_find_one_image_in_bigger_tex_block__oxa_path():
    content = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{oxa:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E.1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    updated = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E.1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )

    assert summary.content == updated
    assert summary.remote_paths == ["oxa:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E.1"]
    assert summary.local_paths == ["images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E.1"]


def test_find_two_images_in_tex__block_path():
    content = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
        \\includegraphics[width=0.7\\linewidth]{block:aaaaabbbbbcccccddddd/xxxxxyyyyyccccc11111/9}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    updated = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
        \\includegraphics[width=0.7\\linewidth]{images/aaaaabbbbbcccccddddd_xxxxxyyyyyccccc11111_9}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )

    assert summary.content == updated
    assert len(summary.remote_paths) == 2
    assert len(summary.local_paths) == 2

    assert summary.remote_paths == [
        "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1",
        "block:aaaaabbbbbcccccddddd/xxxxxyyyyyccccc11111/9",
    ]

    assert summary.local_paths == [
        "images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1",
        "images/aaaaabbbbbcccccddddd_xxxxxyyyyyccccc11111_9",
    ]


def test_find_two_images_in_tex__mixed_paths():
    content = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
        \\includegraphics[width=0.7\\linewidth]{oxa:aaaaabbbbbcccccddddd/xxxxxyyyyyccccc11111.9}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    updated = """
        lorem ipsum anythingim
        \\subsection*{Heading 3}
        \\begin{center}
        \\includegraphics[width=0.7\\linewidth]{images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1}
        \\end{center}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
        \\includegraphics[width=0.7\\linewidth]{images/aaaaabbbbbcccccddddd_xxxxxyyyyyccccc11111.9}
        \\subsection*{Heading 3}
        lorem ipsum anythingim
    """

    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )

    assert summary.content == updated
    assert len(summary.remote_paths) == 2
    assert len(summary.local_paths) == 2

    assert summary.remote_paths == [
        "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1",
        "oxa:aaaaabbbbbcccccddddd/xxxxxyyyyyccccc11111.9",
    ]

    assert summary.local_paths == [
        "images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1",
        "images/aaaaabbbbbcccccddddd_xxxxxyyyyyccccc11111.9",
    ]


def test_patch_one_image_in_tex_block__block_path():
    content = """
        abc
        \\includegraphics[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1}
        xyz
    """

    updated = """
        abc
        \\includegraphics[width=0.7\\linewidth]{images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1}
        xyz
    """
    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )

    assert summary.content == updated
    assert summary.remote_paths == ["block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1"]
    assert summary.local_paths == ["images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E_1"]


def test_patch_one_image_in_tex_block__oxa_path():
    content = """
        abc
        \\includegraphics[width=0.7\\linewidth]{oxa:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E.1}
        xyz
    """

    updated = """
        abc
        \\includegraphics[width=0.7\\linewidth]{images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E.1}
        xyz
    """
    summary = utils.patch_local_images_from_tex_block(
        [utils.INLINE_IMAGE_BLOCK_REGEX, utils.INLINE_IMAGE_OXA_REGEX], content
    )

    assert summary.content == updated
    assert summary.remote_paths == ["oxa:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E.1"]
    assert summary.local_paths == ["images/VYoEVcBJZ5lZ0MW7cYh7_UtUKWdGQQquIXAxWmN9E.1"]


def test_output_image_regex_is_matched():
    content = (
        "abc"
        "\\includegraphics[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1-output-3}"
        "xyz"
    )

    match = re.search(utils.OUTPUT_IMAGE_BLOCK_REGEX, content)
    assert match is not None
    assert match[0] == content
    assert match[1] == "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1-output-3"


def test_output_image_regex_not_matched():
    content = "\\includegraphics[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1}"

    match = re.search(utils.OUTPUT_IMAGE_BLOCK_REGEX, content)
    assert match is None


def test_output_svg_regex_is_matched():
    content = (
        "abc"
        "\\includesvg[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1-output-3}"
        "xyz"
    )

    match = re.search(utils.OUTPUT_SVG_BLOCK_REGEX, content)
    assert match is not None
    assert match[0] == content
    assert match[1] == "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1-output-3"


def test_output_svg_regex_not_matched():
    content = "\\includesvg[width=0.7\\linewidth]{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E/1}"

    match = re.search(utils.OUTPUT_SVG_BLOCK_REGEX, content)
    assert match is None


def test_cite_regex():
    content = r"""single citation in a block \citep{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E}
two copies of the same citation (\cite{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9F}) within the same \citet{block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9G} block
    """
    match = re.findall(utils.INLINE_CITATION_BLOCK_REGEX, content)
    assert len(match) == 3
    assert match[0] == "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9E"
    assert match[1] == "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9F"
    assert match[2] == "block:VYoEVcBJZ5lZ0MW7cYh7/UtUKWdGQQquIXAxWmN9G"


def test_figure_regex():
    content = r"""\\begin{figure}[h]
        \\centering
        \\includegraphics[width=0.7\linewidth]{block:V1qG9ZexyVrBGNsaz5Rh/OZqqWnecItn4CkZPDCy6/1}
        \\caption{block:V1qG9ZexyVrBGNsaz5Rh/OZqqWnecItn4CkZPDCy6/1.caption}
        \\label{some-ting-1}
    \\end{center}"""

    match = re.findall(utils.CAPTION_COMMAND_BLOCK_REGEX, content, flags=re.S)
    assert match
    assert len(match) == 1
    assert match[0] == "block:V1qG9ZexyVrBGNsaz5Rh/OZqqWnecItn4CkZPDCy6/1.caption"


def test_figure_label():
    content = r"""\\begin{figure}[h]
        \\centering
        \\includegraphics[width=0.7\linewidth]{block:V1qG9ZexyVrBGNsaz5Rh/OZqqWnecItn4CkZPDCy6/1}
        \\caption{block:V1qG9ZexyVrBGNsaz5Rh/OZqqWnecItn4CkZPDCy6/1.caption}
        \\label{some-ting-1}
    \\end{center}"""

    match = re.findall(utils.LABEL_COMMAND_REGEX, content, flags=re.S)
    assert match
    assert len(match) == 1
    assert match[0] == "some-ting-1"


def test_parse_cite_tag_from_version():
    local_tag, plain_tag = utils.parse_cite_tag_from_bibtex(
        """@article{Cockett2016Pixels,
        author = {Cockett, Rowan and Heagy, Lindsey J. and Oldenburg, Douglas W.},
        journal = {The Leading Edge},
        number = {8},
        year = {2016},
        month = {8},
        pages = {703--706},
        title = {Pixels and their neighbors: Finite volume},"""
    )
    assert "Cockett2016Pixels" in plain_tag
    assert re.match(r"^Cockett2016Pixels_[0-9a-zA-Z]+", local_tag)


def test_find_groups_in_content():
    pass


@pytest.fixture(
    params=[
        ("", ""),
        ("hello world", "hello world"),
        ("he\\ llo world", r"he{\textbackslash}~llo world"),
        ("he\\llo world", r"he{\textbackslash}llo world"),
        ("he\\llo wor\\ld", r"he{\textbackslash}llo wor{\textbackslash}ld"),
        ("hello~world", r"hello{\textasciitilde}world"),
        ("hello&world", r"hello\&world"),
        ("hello%world", r"hello\%world"),
        ("hello$world", r"hello\$world"),
        ("hello#world", r"hello\#world"),
        ("hello_world", r"hello\_world"),
        ("hello{world", r"hello\{world"),
        ("hello}world", r"hello\}world"),
        ("hello^world", r"hello\^world"),
    ],
    name="to_escape",
)
def _to_escape(request):
    return request.param


def test_escape_latex(to_escape):
    example, escaped = to_escape
    assert utils.escape_latex(example) == escaped


def test_local_markers_can_be_used_in_set_based_on_marker():
    deduped = list(
        set(
            [
                LocalMarker(
                    marker="a", local_path="qwe", remote_path="daa", content="ghj"
                ),
                LocalMarker(
                    marker="b", local_path="qwsde", remote_path="da6a", content="g4hj"
                ),
                LocalMarker(
                    marker="a", local_path="qwhe", remote_path="da8a", content="gh32j"
                ),
            ]
        )
    )

    assert len(deduped) == 2
    assert deduped[0].marker == "a" or deduped[0].marker == "b"
    assert deduped[1].marker == "a" or deduped[1].marker == "b"



import logging
import os
from re import template
from typing import Dict, Tuple, cast
from zipfile import ZipFile

import pkg_resources
from curvenote_template import TemplateLoader, TemplateOptions, TemplateRenderer

from ..client import Session
from ..models import BlockFormat
from ..utils import download

TEMPLATES_PATH = pkg_resources.resource_filename("curvenote", "latex/templates")
DEFAULT_TEMPLATE_PATH = os.path.join(TEMPLATES_PATH, "default")


class OnlineTemplateLoader(TemplateLoader):
    def __init__(self, template_location: str):
        super().__init__(template_location)

    def initialise_from_template_api(
        self, session: Session, template_name: str
    ) -> Tuple[TemplateOptions, TemplateRenderer]:
        logging.info("Writing to target folder: %s", self._target_folder)

        logging.info("Looking up template %s", template_name)
        try:
            link = session.get_template_download_link(template_name)
        except ValueError as err:
            logging.error("could not download template %s", template_name)
            raise ValueError(f"could not download template: {template_name}") from err

        # fetch template to local folder
        logging.info("Found template, downloading...")
        zip_filename = os.path.join(
            self._target_folder, f"{template_name}.template.zip"
        )
        download(link, zip_filename)

        # unzip
        logging.info("Download complete, unzipping...")
        with ZipFile(zip_filename, "r") as zip_file:
            zip_file.extractall(self._target_folder)
        logging.info("Unzipped to %s", self._target_folder)
        os.remove(zip_filename)
        logging.info("Removed %s", zip_filename)

        # success -- update members
        self._template_name = template_name
        renderer = TemplateRenderer()
        renderer.use_from_folder(self._target_folder)

        return TemplateOptions(self._target_folder), renderer
